/*
 * @Author: xun
 * @Date: 2024-04-03 16:02:39
 * @LastEditors: xun
 * @LastEditTime: 2025-02-28 15:24:33
 * @Description: 
 * Copyright (c) 2024 by CCYH, All Rights Reserved. 
 */
#ifndef _NN_UTILS_H_
#define _NN_UTILS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdint.h>

/**
 * @brief 错误码补充
 * 
 */
typedef enum _nn_status
{
    NN_ERROR_QUANTIZE                 = -21,
    NN_ERROR_NORMALIZE                = -20,
    NN_MALLOC_BUFFER                  = -19,
    NN_MALLOC_OUT_PARAM               = -18,
    NN_GET_OUTPUT_CNT                 = -17,
    NN_ERROR_INFER_TIME               = -16,
    NN_SUCCESS                        =  0
} nn_status_e;

/**
 * @brief 将错误码转换成字符串
 * 
 * @param IN  i_errno错误码
 * @return char * 转换后的字符串
 */
char *errno2str(int i_errno);


/**
 * @brief 打印字符串
 * 
 */
void print_msg(const char *format, ...) __attribute__ ((format(printf,1,2)));


/**
 * @brief 将文件以二进制的形式读到内存
 * 
 * @param IN  name 文件名
 * @param OUT dst  读到的文件内容
 * @return unsigned int 读到的文件大小
 */
unsigned int load_binary(const char *name, void *dst);


/**
 * @brief 将tensor文件读到内存，tensor文件格式为每一行一个浮点类型的数
 * 
 * @param IN  name 文件名
 * @param IN  size 待读取的个数
 * @param OUT dst  读到的文件内容
 * @return unsigned int 读到的个数
 */
unsigned int load_text_fp32(const char *name, const uint32_t size, void *dst);


/**
 * @brief 将数据以二进制的形式写到文件中
 * 
 * @param IN  name 文件名
 * @param IN  data 待保存的数据
 * @param OUT size 待保存数据的大小
 * @return unsigned int 实际写到文件中的大小
 */
unsigned int save_to_binary(const char *name, void *data, unsigned int size);


/**
 * @brief 将U8类型的数据写到文件中
 * 
 * @param IN  name 文件名
 * @param IN  data 待保存的数据
 * @param OUT size 待保存数据的大小
 * @return unsigned int 实际写到文件中的大小
 */
unsigned int save_u8_to_text(const char *name, void *data, unsigned int size);


/**
 * @brief 将float类型的数据写到文件中
 * 
 * @param IN  name 文件名
 * @param IN  data 待保存的数据
 * @param OUT size 待保存数据的大小
 * @return unsigned int 实际写到文件中的大小
 */
unsigned int save_fp32_to_text(const char *name, float *data, unsigned int size);


/**
 * @brief 利用 ftell 函数获取文件大小
 * 
 * @param IN  name 文件名
 * @return unsigned int 文件大小
 */
unsigned int get_file_size(const char *name);


/**
 * @brief 解码JPEG文件，得到BMP格式（U8）的结果
 * 
 * @param IN  input_name 输入文件名数组（一个网络可能有多个输入）
 * @param IN  w 输入图片的宽
 * @param IN  h 输入图片的高
 * @param IN  c 输入图片的通道数
 * @param IN  input_count 网络输入个数
 * @param OUT output_buffer 将输入文件解码后，存放得到的结果指针
 * @return int 处理后的状态码，0表示成功，其他表示失败
 */
int get_jpeg_data(char **input_name, uint32_t w, uint32_t h, uint32_t c, \
                const uint16_t input_count, void **output_buffer);


/**
 * @brief 解码TENSOR文件，得到float格式的结果
 * 
 * @param IN  input_name 输入文件名数组（一个网络可能有多个输入）
 * @param IN  input_count 网络输入个数
 * @param IN  input_size  网络输入的大小
 * @param OUT output_buffer 将输入文件解码后，存放得到的结果指针
 * @return int 处理后的状态码，0表示成功，其他表示失败
 */
int get_tensor_data(char **input_name, const int input_count,\
                    const uint32_t *input_size, void **output_buffer);

/**
 * @brief 解码二进制文件，得到U8格式的结果
 * 
 * @param IN  input_name 输入文件名数组（一个网络可能有多个输入）
 * @param IN  input_count 网络输入个数
 * @param IN  input_size  网络输入的大小
 * @param OUT output_buffer 将输入文件解码后，存放得到的结果指针
 * @return int 处理后的状态码，0表示成功，其他表示失败
 */
int get_binary_data(char **input_name, const int input_count,\
                    const uint32_t *input_size, void **output_buffer);


/**
 * @brief 在字符串中查找第一次出现的子字符串 orig，并将其替换为字符串 rep，最终结果存储在 output 中
 * 
 * @param IN  str  要查找和替换的原始字符串
 * @param IN  orig 要查找并替换的子字符串
 * @param IN  rep  替换 orig 的字符串
 * @param OUT output 存储结果的字符串
 * @return NULL
 */
void find_replace(const char* str, char* orig, char* rep, char* output);


/**
 * @brief 查找和替换文件路径中的文件扩展名，比如将 ".txt" 替换为 ".csv"
 * 
 * @param IN  str  要查找和替换的原始字符串
 * @param IN  orig 要查找并替换的子字符串
 * @param IN  rep  替换 orig 的字符串
 * @param OUT output 存储结果的字符串
 * @return NULL
 */
void find_replace_extension(char *str, char *orig, char *rep, char *output);

#ifdef __cplusplus
}
#endif

#endif  // !_NN_UTILS_H_

